
/*
Copyright (C) 2001-200
by Justin Woods <justin@woodsmedia.com>
and David Sowder <david@sowder.edu>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

// $Id: webstamp.inc 466 2007-03-25 23:56:19Z davids $

function get_needed_tags_list($workingset) {
  preg_match_all( "/\{(=?_?[A-Z][^}]+)\}/", $workingset, $m );
  $m = $m[ 1 ];
  if( !is_array( $m )) {
    return array();
  }
  reset( $m );
  $result = array();
  while( list( $k, $v ) = each( $m )) {
    if( !isset( $result[ $v ] )) {
      $result[ $v ] = $v;
    }
  }
  if( count( $result )) {
    return $result;
  } else {
    return array();
  }
}

function get_directory_steps_to_libs_dir( $directory ) {
  $dir_list = array();
  while(  $directory = dirname( $directory )) {
    $dir_list[] = $directory;
    if( $directory == '/' ) {
      break;
    }
    else if( file_exists( $directory . '/.libs' )) {
      $dir_list[] = $directory . '/.libs';
      break;
    }
  }
  return array_reverse( $dir_list );
}

function get_reldir_matched_url( $reldir,$url ) {
  $reldir = $reldir . '/';
  if( !strncmp( $reldir, './', 2 )) {
    $reldir = substr( $reldir, 2 );
  }
  while( !strncmp( $reldir, '../', 3 )) {
    $reldir = substr( $reldir, 3 );
    $url = dirname( $url ) . '/';
  }
  if( !strncmp( $url, '//', 2 )) {
    $url = substr( $url, 1 );
  }
  return $url;
}

function get_file_list( $directory, $prefix = '' ) {
  global $OURLIBS;

  $prefix = !empty( $prefix ) ? '\\'.$prefix : '';
  $d = dir( $directory );
  $file_list = array();
  while( $file = $d->read() ) {
    if( ereg( '^' . $prefix . '_?[A-Z][A-Za-z0-9_]*$', $file )
        && strcmp( $file, 'CVS' )  
        && strcmp( $file, '^MAIN' )
        && is_readable( realpath( $directory . '/' . $file ))) {
      //It's stupid we have to build the array this way, but there IS a
      //bug in this version of PHP (4.0.3)
      $file = str_replace( '^', '', $file );
      $file_list = array_merge( $file_list, array( 0 => $file ));
      // $file_list[] = $file;  //I'd much rather build the array this way.
    }
  }
  $d -> close();
  $d = dir( $OURLIBS );
  while( $file = $d->read() ) {
    if( ereg('^=_?[A-Z][A-Za-z0-9_]*$', $file) 
        && strcmp( $file, 'CVS' )  
        && strcmp( $file, '^MAIN' )
        && is_readable( realpath( $OURLIBS . '/' . $file ))) {
      // It's stupid we have to build the array this way, but there IS a
      // bug in this version of PHP (4.0.3)
      $file = str_replace( '^', '', $file );
      $file_list = array_merge( $file_list, array( 0 => $file ));
      // $file_list[] = $file;  //I'd much rather build the array this way.
    }
  }
  $d -> close();
  return $file_list;
}

function parse_dynamic_tag( $tag, $tagfile ) {
  global $ignore_http_get_vars, $ignore_http_post_vars;

  srand( (double) microtime() * 1000000 );
  $tagfile_contents = '';
  $funcname = "dynamic${tag}_" . rand( 0, 99999999 );
  $tagfile_contents .= @implode( '', file( $tagfile ));
  // Below, we don't use newlines so all the 'overhead' code generated by webstamp should hopefully not impact eval'd code line numbers anymore
  $funcdef_contents = "function $funcname() {";
  $funcdef_contents .= '  global $OURDIR, $OURCACHE, $OURDATA, $OURLIBS, $OURURL, $OURGFXURL;';
  $funcdef_contents .= '  global $ROOTDIR, $ROOTCACHE, $ROOTDATA, $ROOTLIBS, $ROOTURL, $ROOTGFXURL;';
  $funcdef_contents .= '  global $THISDIR, $THISURL;';
  if( !isset( $_GET ) && isset( $GLOBALS[ 'HTTP_GET_VARS' ] )) {
    $_GET = $GLOBALS[ 'HTTP_GET_VARS' ];
  }
  if( isset( $_GET )) {
    $funcdef_contents .= '  extract( $_GET );';
  } else if( !isset( $ignore_http_get_vars ) || $ignore_http_get_vars == false ) {
    echo "\$_GET (nor \$HTTP_GET_VARS) was not made available by\n";
    echo "PHP.  Webstamp expects it to be available for fields included in\n";
    echo "a GET method form.  If it's lack of availability is intentional,\n";
    echo "then set the \$ignore_http_get_vars variable in your root\n";
    echo "siteconfig.inc to true.\n";
    die();
  }
  if( !isset( $_POST ) && isset( $GLOBALS[ 'HTTP_POST_VARS' ] )) {
    $_POST = $GLOBALS[ 'HTTP_POST_VARS' ];
  }
  if( isset( $_POST )) {
    $funcdef_contents .= '  extract( $_POST );';
  } else if( !isset( $ignore_http_post_vars ) || $ignore_http_post_vars == false ) {
    echo "\$_POST (nor \$HTTP_POST_VARS) was not made available by PHP.  Webstamp\n";
    echo "expects it to be available for fields included in a POST method form. \n";
    echo "If it's lack of availability is intentional, then set the\n";
    echo "\$ignore_http_post_vars variable in your root siteconfig.inc to true.\n";
    die();
  }
  $funcdef_contents .= $tagfile_contents;
  $funcdef_contents .= "}\n";
  eval( $funcdef_contents );
  ob_start();
  eval( "\$result = $funcname();" );
  $file_contents = ob_get_contents();
  ob_end_clean();
  return( $file_contents );
}

function parse_file_array( $file_array, $workingset, $OURLIBS, $directory, $prefix = '' ) {
  if( count($file_array) > 0 ) {
    while( list($i, $tag) = each($file_array) ) {
      if( $tag[0] == '_' ) {
        $file_contents = parse_dynamic_tag( $tag, $directory . '/' . $prefix . $tag );
      } else if( $tag[0] == '=' && $tag[1] == '_' ) {
        $shavedtag = substr( $tag, 1, strlen( $tag ));
        $file_contents = parse_dynamic_tag( $shavedtag, $OURLIBS . '/' . $tag );
      } else if( $tag[0] == '=' ) {
        $file_contents = @implode('', file( $OURLIBS . '/' . $tag ));
      } else {
        $file_contents = @implode('', file( "$directory/$prefix$tag" ));
      }
      $search = '{' . $tag . '}';
      $workingset = str_replace( $search, $file_contents, $workingset );
    }
  }
  return $workingset;
}

function process( $OURLIBS, $directory, $workingset, $prefix = '' ) {
  $files = get_file_list( $directory, $prefix );
  $recursive_list = array();
  do {
    unset( $todo_list );
    $tags_list = get_needed_tags_list( $workingset );
    $tags_list = array_unique( $tags_list );
    $unfound_list = array_diff( $tags_list, $files );
    $tags_list = array_intersect( $tags_list, $files );
    while( list( $i, $tag ) = each( $tags_list )) {
      if( $tag[0] != '_' && $tag[0] != '=' ) {
        $file_contents = implode( '', file( $directory.'/'.$prefix.$tag ));
        if( strstr( $file_contents, '{' . $tag . '}' )) {
          $recursive_list = array_unique( array_merge( $recursive_list, array( 0 => $tag )));
        }
      }
    }
    $todo_list = array_diff( $tags_list, $recursive_list );
    $workingset = parse_file_array( $todo_list, $workingset, $OURLIBS, $directory, $prefix );
  } while( count( $todo_list ) > 0 );
  $workingset = parse_file_array( $recursive_list, $workingset, $OURLIBS, $directory, $prefix );
  unset( $unfound_list, $recursive_list, $todo_list );
  return $workingset;
}

function gzip_accepted( $ACCEPTED_ENCODINGS ) {
  if( strpos( $ACCEPTED_ENCODINGS, 'gzip' ) === false ) {
    return false;
  }
  if( strpos( $ACCEPTED_ENCODINGS, 'x-gzip' ) === false ) {
    $encoding = 'gzip';
  } else {
    $encoding = 'x-gzip';
  }
  return $encoding;
}

?>
